@@ -9,8 +9,6 @@ Information sourced from u-blox Interface Specifications © 2013-2021, u-blox AG
 """
 
 UBX_HDR = b"\xb5\x62"
-INPUT = 1
-OUTPUT = 0
 GET = 0
 SET = 1
 POLL = 2
@@ -25,6 +23,18 @@ GNSSLIST = {
     6: "GLONASS",
 }
 
+# scaling factor constants
+SCAL9 = 1e-9  # 0.000000001
+SCAL8 = 1e-8  # 0.00000001
+SCAL7 = 1e-7  # 0.0000001
+SCAL6 = 1e-6  # 0.000001
+SCAL5 = 1e-5  # 0.00001
+SCAL4 = 1e-4  # 0.0001
+SCAL3 = 1e-3  # 0.001
+SCAL2 = 1e-2  # 0.01
+SCAL1 = 1e-1  # 0.1
+SCALROUND = 8  # number of dp to round scaled attributes to
+
 # **************************************************
 # THESE ARE THE UBX PROTOCOL PAYLOAD ATTRIBUTE TYPES
 # **************************************************
@@ -52,9 +62,14 @@ U6 = "U006"  # Unsigned Int 6 bytes
 U7 = "U007"  # Unsigned Int 7 bytes
 U8 = "U008"  # Unsigned Int 8 bytes
 U9 = "U009"  # Unsigned Int 9 bytes
+U10 = "U010"  # Unsigned Int 10 bytes
+U11 = "U011"  # Unsigned Int 11 bytes
 U12 = "U012"  # Unsigned Int 12 bytes
 U20 = "U020"  # Unsigned Int 20 bytes
 U22 = "U022"  # Unsigned Int 22 bytes
+U23 = "U023"  # Unsigned Int 23 bytes
+U24 = "U024"  # Unsigned Int 24 bytes
+U32 = "U032"  # Unsigned Int 32 bytes
 U40 = "U040"  # Unsigned Int 40 bytes
 U64 = "U064"  # Unsigned Int 64 bytes
 X1 = "X001"  # Bitfield 1 byte
@@ -66,46 +81,6 @@ X24 = "X024"  # Bitfield 24 bytes
 R4 = "R004"  # Float (IEEE 754) Single Precision 4 bytes
 R8 = "R008"  # Float (IEEE 754) Double Precision 8 bytes
 
-VALID_TYPES = (
-    A256,
-    C2,
-    C6,
-    C10,
-    C30,
-    C32,
-    CH,
-    E1,
-    E2,
-    E4,
-    I1,
-    I2,
-    I4,
-    I8,
-    L,
-    R4,
-    R8,
-    U1,
-    U2,
-    U3,
-    U4,
-    U5,
-    U6,
-    U7,
-    U8,
-    U9,
-    U12,
-    U20,
-    U22,
-    U40,
-    U64,
-    X1,
-    X2,
-    X4,
-    X6,
-    X8,
-    X24,
-)
-
 # ***********************************************
 # THESE ARE THE UBX PROTOCOL CORE MESSAGE CLASSES
 # ***********************************************
@@ -125,6 +100,7 @@ UBX_CLASSES = {
     b"\x21": "LOG",  # Logging Messages: Log creation, deletion, info and retrieval
     b"\x27": "SEC",  # Security Feature Messages
     b"\x28": "HNR",  # High Rate Navigation Messages
+    b"\x29": "NAV2",  # Navigation 2 Results: Position, Speed, Time, Acc, Heading, DOP, SVs used
     b"\xf0": "NMEA-Standard",  # Standard NMEA Messages: Used for message rate configuration via CFG-MSG
     b"\xf1": "NMEA-Proprietary",  # Proprietary NMEA Messages: Used for message rate configuration via CFG-MSG
     b"\xf5": "RTCM",  # RTCM Messages: Used for message rate configuration via CFG-MSG
@@ -158,12 +134,15 @@ UBX_MSGIDS = {
     b"\x06\x70": "CFG-DGNSS",
     b"\x06\x61": "CFG-DOSC",
     b"\x06\x85": "CFG-DYNSEED",
+    b"\x06\x12": "CFG-EKF",
     b"\x06\x56": "CFG-ESFALG",
     b"\x06\x4c": "CFG-ESFA",
     b"\x06\x4d": "CFG-ESFG",
     b"\x06\x82": "CFG-ESFWT",
+    b"\x06\x29": "CFG-ESFGWT",
     b"\x06\x60": "CFG-ESRC",
     b"\x06\x84": "CFG-FIXSEED",
+    b"\x06\x0e": "CFG-FXN",  # deprecated, use CFG-PM2
     b"\x06\x69": "CFG-GEOFENCE",
     b"\x06\x3e": "CFG-GNSS",
     b"\x06\x5c": "CFG-HNR",
@@ -174,6 +153,7 @@ UBX_MSGIDS = {
     b"\x06\x24": "CFG-NAV5",
     b"\x06\x23": "CFG-NAVX5",
     b"\x06\x17": "CFG-NMEA",  # NB: 3 versions of this
+    b"\x06\x22": "CFG-NVS",
     b"\x06\x1e": "CFG-ODO",
     b"\x06\x3b": "CFG-PM2",
     b"\x06\x86": "CFG-PMS",
@@ -261,10 +241,10 @@ UBX_MSGIDS = {
     b"\x13\x00\x04": "MGA-GPS-HEALTH",
     b"\x13\x00\x05": "MGA-GPS-UTC",
     b"\x13\x00\x06": "MGA-GPS-IONO",
-    b"\x13\x40\x00": "MGA-INI-POS_XYZ",
-    b"\x13\x40\x01": "MGA-INI-POS_LLH",
-    b"\x13\x40\x10": "MGA-INI-TIME_UTC",
-    b"\x13\x40\x11": "MGA-INI-TIME_GNSS",
+    b"\x13\x40\x00": "MGA-INI-POS-XYZ",
+    b"\x13\x40\x01": "MGA-INI-POS-LLH",
+    b"\x13\x40\x10": "MGA-INI-TIME-UTC",
+    b"\x13\x40\x11": "MGA-INI-TIME-GNSS",
     b"\x13\x40\x20": "MGA-INI-CLKD",
     b"\x13\x40\x21": "MGA-INI-FREQ",
     b"\x13\x40\x30": "MGA-INI-EOP",
@@ -300,6 +280,7 @@ UBX_MSGIDS = {
     b"\x01\x31": "NAV-DGPS",
     b"\x01\x04": "NAV-DOP",
     b"\x01\x3d": "NAV-EELL",
+    b"\x01\x40": "NAV-EKFSTATUS",  # deprecated, use ESF-STATUS and ESF-MEAS
     b"\x01\x61": "NAV-EOE",
     b"\x01\x39": "NAV-GEOFENCE",
     b"\x01\x13": "NAV-HPPOSECEF",
@@ -310,7 +291,7 @@ UBX_MSGIDS = {
     b"\x01\x01": "NAV-POSECEF",
     b"\x01\x02": "NAV-POSLLH",
     b"\x01\x07": "NAV-PVT",
-    b"\x01\x3c": "NAV-RELPOSNED",
+    b"\x01\x3c": "NAV-RELPOSNED",  # two versions
     b"\x01\x10": "NAV-RESETODO",
     b"\x01\x35": "NAV-SAT",
     b"\x01\x32": "NAV-SBAS",
@@ -330,6 +311,29 @@ UBX_MSGIDS = {
     b"\x01\x11": "NAV-VELECEF",
     b"\x01\x12": "NAV-VELNED",
     # ***************************************************************
+    # Navigation 2 messages
+    # ***************************************************************
+    b"\x29\x22": "NAV2-CLOCK",
+    b"\x29\x36": "NAV2-COV",
+    b"\x29\x04": "NAV2-DOP",
+    b"\x29\x61": "NAV2-EOE",
+    b"\x29\x09": "NAV2-ODO",
+    b"\x29\x01": "NAV2-POSECEF",
+    b"\x29\x02": "NAV2-POSLLH",
+    b"\x29\x07": "NAV2-PVT",
+    b"\x29\x35": "NAV2-SAT",
+    b"\x29\x32": "NAV2-SBAS",
+    b"\x29\x43": "NAV2-SIG",
+    b"\x29\x03": "NAV2-STATUS",
+    b"\x29\x24": "NAV2-TIMEBDS",
+    b"\x29\x25": "NAV2-TIMEGAL",
+    b"\x29\x23": "NAV2-TIMEGLO",
+    b"\x29\x20": "NAV2-TIMEGPS",
+    b"\x29\x26": "NAV2-TIMELS",
+    b"\x29\x21": "NAV2-TIMEUTC",
+    b"\x29\x11": "NAV2-VELECEF",
+    b"\x29\x12": "NAV2-VELNED",
+    # ***************************************************************
     # Receiver Management messages
     # ***************************************************************
     b"\x02\x61": "RXM-IMES",
@@ -345,6 +349,8 @@ UBX_MSGIDS = {
     # ***************************************************************
     # Security messages
     # ***************************************************************
+    b"\x27\x09": "SEC-SIG",
+    b"\x27\x10": "SEC-SIGLOG",
     b"\x27\x01": "SEC-SIGN",
     b"\x27\x03": "SEC-UNIQID",
     # ***************************************************************
@@ -358,7 +364,7 @@ UBX_MSGIDS = {
     b"\x0d\x03": "TIM-TM2",
     b"\x0d\x12": "TIM-TOS",
     b"\x0d\x01": "TIM-TP",
-    b"\x0d\x15": "TIM-VCOCAL",
+    b"\x0d\x15": "TIM-VCOCAL",  # 2 versions of SET
     b"\x0d\x06": "TIM-VRFY",
     # ***************************************************************
     # Firmware update messages
